﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NHibernateWorkshop.Domain;
using NHibernate.Criterion;
using NHibernate;

namespace NHibernateWorkshop.DataAccess
{
    public class Repository
    {
        private ISession _session;
        /// <summary>
        /// Initializes a new instance of the Repository class.
        /// </summary>
        public Repository(ISession session)
        {
            _session = session;
        }

        public Customer GetCustomerById(int customerId)
        {
            return _session.Get<Customer>(customerId);
        }

        public IEnumerable<Customer> GetCustomerByFirstnameViaHql(string firstname)
        {
            return _session.CreateQuery("from Customer c where c.Firstname=:fname")
                .SetString("fname", firstname)
                .List<Customer>();
        }

        public IEnumerable<Customer> GetCustomerByExample(Customer customer)
        {
            return _session.CreateCriteria<Customer>()
                .Add(Example.Create(customer))
                .List<Customer>();
        }

        public void UpdateCustomer(Customer customer)
        {

            using (var tx = _session.BeginTransaction())
            {
                try
                {
                    _session.Update(customer);
                    tx.Commit();
                }
                catch (Exception ex)
                {
                    tx.Rollback();
                    throw;
                }
            }
                       

        }

        public void UpdateCustomerWithException(Customer customer)
        {

            using (var tx = _session.BeginTransaction())
            {
                try
                {
                    _session.Update(customer);
                    throw new HibernateException();
                    tx.Commit();
                }
                catch (Exception ex)
                {
                    tx.Rollback();
                    throw;
                }

            }

        }


        public void DeleteCustomer(Customer customer)
        {

            using (var tx = _session.BeginTransaction())
            {
                try
                {
                    _session.Delete(customer);
                    tx.Commit();
                }
                catch (HibernateException)
                {
                    tx.Rollback();
                    throw;
                }
            }

                        

        }

        public void AddCustomer(Customer customer)
        {

            using (var tx = _session.BeginTransaction())
            {

                try
                {
                    _session.Save(customer);
                    tx.Commit();
                }
                catch (HibernateException)
                {
                    tx.Rollback();
                    throw;
                }
                
            }
            
            
        }


        

        public IEnumerable<Customer> GetCustomerByFirstnameViaCriteriaApi(string firstname)
        {
            return _session.CreateCriteria<Customer>()
                .Add(Restrictions.Eq("Firstname", firstname))
                .List<Customer>();
        }

    }
}
