﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using NHibernateWorkshop.DataAccess;
using NHibernateWorkshop.Domain;
using NHibernate;

namespace NHibernateWorkshop.Tests
{
    [TestFixture]
    public class WoorkshopTests
    {
        private ISessionFactory _sessionFactory;
        [FixtureSetUp]
        public void _TestFixtureSetUp()
        {

            HibernatingRhinos.NHibernate.Profiler.Appender.NHibernateProfiler.Initialize();

            NHibernate.Cfg.Configuration cfg = new NHibernate.Cfg.Configuration();
            cfg.Configure();

            _sessionFactory = cfg.BuildSessionFactory();
        }

        [Test]
        public void CanDeleteCustomer()
        {
            Repository repository = new Repository(_sessionFactory.OpenSession());

            Customer customer = new Customer() { Firstname = "Please", Lastname = "DeleteMe" };

            repository.AddCustomer(customer);

            Assert.IsNotNull(repository.GetCustomerById(customer.Id));

            repository.DeleteCustomer(customer);
            Assert.IsFalse(customer.Id == 0);
            Assert.IsNull(repository.GetCustomerById(customer.Id));
        }

        [Test]
        public void CanSaveNewCustomer()
        {
            Repository repository = new Repository(_sessionFactory.OpenSession());
            Customer customer = new Customer() { Firstname = "Johan", Lastname = "Testme!" };

            Assert.IsTrue(customer.Id == 0);
            repository.AddCustomer(customer);
            Assert.IsTrue(customer.Id != 0);

        }

      

        [Test]
        public void CanUpdateCustomer()
        {
            Repository repository = new Repository(_sessionFactory.OpenSession());
            var customer = repository.GetCustomerById(1);
            customer.Lastname = customer.Lastname + "ZZZ";

            repository.UpdateCustomer(customer);

            var updatedCustomer = repository.GetCustomerById(1);
            Assert.AreEqual(customer.Lastname, updatedCustomer.Lastname);

        }

        [Test]
        public void CanGetCustomerWithOrders()
        {
            Repository repository = new Repository(_sessionFactory.OpenSession());

            var customer = repository.GetCustomerById(1);

            Assert.IsNotNull(customer.Orders);
            Assert.IsNotEmpty(customer.Orders);

        }

        [Test]
        public void CanPreventOldCustomerOverwirtingNewCustomer()
        {
            Repository repository = new Repository(_sessionFactory.OpenSession());

            var firstCustomer = repository.GetCustomerById(9);
            var secondCustomer = repository.GetCustomerById(9);

            firstCustomer.Lastname = firstCustomer.Lastname + "ZZZ";
            secondCustomer.Lastname = secondCustomer.Lastname + "YYY";

            repository.UpdateCustomer(firstCustomer);
            try
            {
                repository.UpdateCustomer(secondCustomer);
            }
            catch (StaleStateException)
            {
                //business logic would tell user they need to reload the record and try again... (or somthing!)
            }

            var updatedCustomer = repository.GetCustomerById(9);

            Assert.AreEqual(firstCustomer.Lastname, updatedCustomer.Lastname);



            
        }



        [Test]
        public void CantUpdateCustomer()
        {
            Repository repository = new Repository(_sessionFactory.OpenSession());
            var customer = repository.GetCustomerById(1);
            customer.Lastname = customer.Lastname + "ZZZ";

            try
            {
                repository.UpdateCustomerWithException(customer);
                Assert.Fail("shouldnt be here");
            }
            catch (HibernateException)
            {
                
            }

            var updatedCustomer = repository.GetCustomerById(1);
            Assert.AreNotEqual(customer.Lastname, updatedCustomer.Lastname);

        }





        [Test]
        public void CanCustomerById()
        {
            Repository repository = new Repository(_sessionFactory.OpenSession());
            var customer = repository.GetCustomerById(1);

            Assert.AreEqual(1, customer.Id);
        }

        [Test]
        public void CanGetCustomerByFirstnameViaHql()
        {
            Repository repository = new Repository(_sessionFactory.OpenSession());

            var customers = repository.GetCustomerByFirstnameViaHql("Steve");
            foreach (var customer in customers)
            {
                Assert.AreEqual("Steve", customer.Firstname);

            }
        }

        [Test]
        public void CanGetCustomerByFirstnameViaCriteriaApi()
        {
            Repository repository = new Repository(_sessionFactory.OpenSession());

            var customers = repository.GetCustomerByFirstnameViaCriteriaApi("Steve");
            foreach (var customer in customers)
            {
                Assert.AreEqual("Steve", customer.Firstname);

            }
        }


        [Test]
        public void CanGetCustomerByFirstnameViaExample()
        {
            Repository repository = new Repository(_sessionFactory.OpenSession());

            Customer exampleCustomer = new Customer() { Firstname = "Steve" };


            var customers = repository.GetCustomerByExample(exampleCustomer);
            foreach (var customer in customers)
            {
                Assert.AreEqual("Steve", customer.Firstname);

            }
        }

        [Test]
        public void CanGetCustomerByFirstnameAndLastnameViaExample()
        {
            Repository repository = new Repository(_sessionFactory.OpenSession());

            Customer exampleCustomer = new Customer() { Firstname = "Steve", Lastname = "Bohlen" };


            var customers = repository.GetCustomerByExample(exampleCustomer);
            foreach (var customer in customers)
            {
                Assert.AreEqual("Steve", customer.Firstname);
                Assert.AreEqual("Bohlen", customer.Lastname);

            }
        }
    }
}
